@@ -148,7 +148,7 @@ intI <- function(i, n, dn, give.dn = TRUE)
 	i0 <- i0 - 1L
     }
     if(!give.dn) i0 else list(i0 = i0, dn = dn)
-}
+} ## {intI}
 
 .ind.prep <- function(xi, intIlist, iDup = duplicated(i0), anyDup = any(iDup))
 {
@@ -173,19 +173,22 @@ intI <- function(i, n, dn, give.dn = TRUE)
       ## actually,  iDup  is rarely needed in calling code
       if(anyDup) list(iDup = iDup, i0i = i0i, i.x = i.x,
 		      jm = unlist(jm), i.xtra = rep.int(i.x, sapply(jm, length))))
-}
-
+} ## {.ind.prep}
 
+##' <description>
+##' Do the ``common things'' for "*gTMatrix" sub-assignment
+##' for 1 dimension, 'margin' ,
+##' <details>
+##' @title Indexing Preparation
+##' @param i "index"
+##' @param margin in {1,2};
+##' @param di = dim(x)	{ used when i is not character }
+##' @param dn = dimnames(x)
+##' @return match(.,.) + li = length of corresponding dimension
+##' difference to .ind.prep(): use 1-indices; no match(xi,..), no dn at end
+##' @author Martin Maechler
 .ind.prep2 <- function(i, margin, di, dn)
-{    ## Purpose: do the ``common things'' for "*gTMatrix" sub-assignment
-    ##		for 1 dimension, 'margin' ,
-    ##		and return match(.,.) + li = length of corresponding dimension
-    ##
-    ## i is "index"; margin in {1,2};
-    ## di = dim(x)	{ used when i is not character }
-
-    ## difference to .ind.prep(): use 1-indices; no match(xi,..), no dn at end
-
+{
     intI(i, n = di[margin], dn = dn[[margin]], give.dn = FALSE)
 }
 
@@ -195,244 +198,32 @@ setMethod("[", signature(x = "TsparseMatrix", i = "index", j = "missing",
 			 drop = "logical"),
 	  function (x, i, j, ..., drop) { ## select rows
 	      na <- nargs()
-	      Matrix.msg("Tsp[i,m,l] : nargs()=", na, .M.level = 2)
-	      if(na == 3)
-		  ## x[i, drop=TRUE/FALSE]  *vector* indexing !
-		  return(.M.vectorSub(x,i))
-
-	      clx <- getClassDef(class(x))
-	      has.x <- !extends(clx, "nsparseMatrix")
-	      x.sym <- extends(clx, "symmetricMatrix")
-	      x.tri <- !x.sym && extends(clx, "triangularMatrix")
-	      gDo <- (x.sym || (x.tri && x@diag == "U"))
-	      if(gDo)
-		  x <- as(x, paste(.M.kind(x, clx), "gTMatrix", sep=''))
-
-	      d <- x@Dim
-	      n <- d[1]
-              ## FIXME: 'm' is of length 'nnz' which can be large;
-              ##       we do only need  which(.$m > 0L) = which(sel) below...
-              ##-> would like C-based  which.match()
-	      ip <- .ind.prep(x@i, intI(i, n = n, dimnames(x)[[1]]))
-	      Di1 <- ip$li
-	      drop.it <- drop && (Di1 == 1L || d[2] == 1L)
-	      if(x.tri) { # triangular
-		  if(Di1 == n && identical(ip$i0, 0:(n-1))) # *full* result
-		      return(if (drop.it) x[1,1] else x)
-		  ## else: result will *not* be triangular
-		  if(!gDo)
-		      x <- as(x, paste(.M.kind(x, clx), "gTMatrix", sep=''))
-	      }
-	      x@Dim[1] <- Di1
-	      if(!is.null(ip$dn)) x@Dimnames[[1]] <- ip$dn
-	      sel <- seq_along(ip$m)[ip$m > 0L] # == which(match(....))
-	      x@i <- ip$m[sel] - 1L
-	      if(ip$anyDup) { ## duplicated rows selected: extend sel
-		  sel <- c(sel, ip$jm)
-		  x@i <- c(x@i, ip$i.xtra)
-	      }
-	      x@j <- x@j[sel]
-	      if (has.x) x@x <- x@x[sel]
-	      if (drop.it) drop(as(x,"matrix")) else x
+	      Matrix.msg("Tsp[i,m,l]: nargs()=", na, .M.level=2)
+	      if(na == 4)
+		  .as.Tsp(as(x,"CsparseMatrix")[i, , drop=drop], noCheck = !drop)
+	      else if(na == 3) ## e.g. M[0] , M[TRUE],	M[1:2]
+		  .M.vectorSub(x,i)
+	      else ## should not happen
+		  stop("Matrix-internal error in <TsparseM>[i,,d]; please report")
 	  })
 
 ## Select columns
 setMethod("[", signature(x = "TsparseMatrix", i = "missing", j = "index",
 			 drop = "logical"),
 	  function (x, i, j, ..., drop) { ## select columns
-	      Matrix.msg("Tsp[m,i,l] : nargs()=", nargs(), .M.level = 2)
-	      clx <- getClassDef(class(x))
-	      has.x <- !extends(clx, "nsparseMatrix")
-	      x.sym <- extends(clx, "symmetricMatrix")
-	      x.tri <- !x.sym && extends(clx, "triangularMatrix")
-	      gDo <- (x.sym || (x.tri && x@diag == "U"))
-	      if(gDo)
-		  x <- as(x, paste(.M.kind(x, clx), "gTMatrix", sep=''))
-
-	      d <- x@Dim
-	      n <- d[2]
-              ## FIXME: 'm' is of length 'nnz' which can be large;
-              ##       we do only need  which(.$m > 0L) = which(sel) below...
-              ##-> would like C-based  which.match()
-	      ip <- .ind.prep(x@j, intI(j, n = n, dimnames(x)[[2]]))
-	      Di2 <- ip$li
-	      drop.it <- drop && (d[1] == 1L || Di2 == 1L)
-	      if(x.tri) { # triangular
-		  if(Di2 == n && identical(ip$i0, 0:(n-1))) # *full* result
-		      return(if (drop.it) x[1,1] else x)
-		  ## else: result will *not* be triangular
-		  if(!gDo)
-		      x <- as(x, paste(.M.kind(x, clx), "gTMatrix", sep=''))
-	      }
-	      x@Dim[2] <- Di2
-	      if(!is.null(ip$dn)) x@Dimnames[[2]] <- ip$dn
-	      sel <- seq_along(ip$m)[ip$m > 0L] # == which(match(....))
-	      x@j <- ip$m[sel] - 1L
-	      if(ip$anyDup) { ## duplicated columns selected: extend sel
-		  sel <- c(sel, ip$jm)
-		  x@j <- c(x@j, ip$i.xtra)
-	      }
-	      x@i <- x@i[sel]
-	      if (has.x) x@x <- x@x[sel]
-	      if (drop.it) drop(as(x,"matrix")) else x
+	      .as.Tsp(as(x,"CsparseMatrix")[, j, drop=drop], noCheck = !drop)
 	  })
 
-
-## [.data.frame has : drop = if (missing(i)) TRUE else length(cols) == 1)
-
-
 setMethod("[", signature(x = "TsparseMatrix",
 			 i = "index", j = "index", drop = "logical"),
 	  function (x, i, j, ..., drop)
-      {
-	  Matrix.msg("Tsp[i,i,l] : nargs()=", nargs(), .M.level = 2)
-	  ## (i,j, drop) all specified
-	  di <- dim(x)
-	  dn <- dimnames(x)
-	  clx <- getClassDef(class(x))
-	  has.x <- !extends(clx, "nsparseMatrix")
-	  isSym <- extends(clx, "symmetricMatrix")
-	  x.tri <- !isSym && extends(clx, "triangularMatrix")
-	  if(isSym) {
-	      isSym <- (length(i) == length(j) && mode(i) == mode(j) &&
-			isTRUE(all(i == j)))# work for i,j NA
-	      ## result will *still* be symmetric --> keep symmetry!
-	      gDo <- !isSym ## result no longer symmetric -> to "generalMatrix"
-	  } else if(x.tri) {
-	      ii <- intI(i, n = di[1], dn= dn[[1]])
-	      ij <- intI(j, n = di[2], dn= dn[[2]])
-	      gDo <- { (x@diag == "U") ||
-		       ## maybe result is no longer triangular
-		       !(identical(ii$i0, ij$i0) && !is.unsorted(ii$i0))
-		   }
-	  }
-	  else gDo <- FALSE
+	  .as.Tsp(as(x,"CsparseMatrix")[i, j, drop=drop], noCheck = !drop))
 
-	  if(gDo) # go via "generalMatrix":
-	      x <- as(x, paste(.M.kind(x, clx), "gTMatrix", sep=''))
+## This is "just for now" -- Thinking of *not* doing this in the future
+.as.Tsp <- function(x, noCheck)
+    if(noCheck || is(x,"sparseMatrix")) as(x, "TsparseMatrix") else x
 
 
-	  if(isSym) { ## has only stored "half" of the indices,
-	      ## OTOH,	i === j, so only need one intI() call
-	      ip1 <- intI(i, n=di[1], dn[[1]]) # -> (i0, dn_1)
-	      anyDup <- any(iDup <- duplicated(ip1$i0))
-	      ip1 <- .ind.prep(x@i, ip1, iDup=iDup, anyDup=anyDup)
-	      ip2 <- {
-		  if(anyDup) list(m = match(x@j, ip1$i0, nomatch=0),
-				  li = ip1$li)
-		  else	     .ind.prep(x@j, ip1, iDup=iDup, anyDup=anyDup)
-	      }
-	      if(!is.null(dn[[2]]))	# fix result colnames
-		  ip2$dn <- dn[[2]][ip1$i0 + 1L]
-
-	  } else {
-	      if(!x.tri) {
-		  ii <- intI(i, n = di[1], dn= dn[[1]])
-		  ij <- intI(j, n = di[2], dn= dn[[2]])
-	      }
-              ## FIXME ip1$m , ip2$m  of length 'nnz' can be large;
-              ## rather use which.match(), or which.match("first" & "second")
-	      ip1 <- .ind.prep(x@i, ii)
-	      ip2 <- .ind.prep(x@j, ij)
-	  }
-	  nd <- c(ip1$li, ip2$li)
-	  x@Dim <- nd
-	  x@Dimnames <- list(ip1$dn, ip2$dn)
-
-	  if(isSym) {
-	      sel <- which(ip1$m  &  ip2$m)## FIXME
-	      ii <- ip1$m[sel] - 1L
-	      jj <- ip2$m[sel] - 1L
-	      if(anyDup) { ## careful algorithm --- TODO: in C
-		  ## keep non-duplicated and "increment" for duplicated ones
-		  ij <- pmin(ii, jj)
-		  jj <- pmax(ii, jj) ; ii <- ij
-		  ## length(ii) == length(jj) == length(sel)  _and_  ii <= jj
-		  ix <- ip1$i.x
-		  for(k in seq_along(ix)) {
-		      ## "recursively" add 1 row+column corresp. iDup[k]
-		      i0 <- ip1$i0i[k] -1L # the (0-ind)column we want to repl
-		      i.x <- ix[k]	# < i0
-		      e1 <- ii == i0
-		      e2 <- jj == i0
-		      j1 <- jj[e1]	# >= ii[e1] == i0
-		      j2 <- ii[e2]	# <= jj[e2] == i0 < i.x
-		      ## now the "diagonal special":
-		      if(any(e1 & e2)) {
-			  ## (e1 & e2)[m] = TRUE  <==>	ii[m] == jj[m] == i0
-			  isD <- e1[e2] # logical of same length as j2
-			  stopifnot(sum(isD) == 1)
-			  j2[isD] <- i.x # instead of i0
-		      }
-		      l1x <- j1 < i.x	# & j12 <- j1[l1x]
-		      j11 <- j1[!l1x]	# those >= i.x
-		      s1 <- sel[e1]
-		      ii  <- c(ii,  j2, j1[l1x],  rep.int(i.x, length(j11)))
-		      jj  <- c(jj,  rep.int(ix[k], length(j2)+sum(l1x)), j11)
-		      sel <- c(sel, sel[e2], s1[l1x], s1[!l1x])
-		      stopifnot(ii <= jj, length(sel) == length(ii))
-		  }
-		  if(x@uplo == "U") { ## i <= j : upper triangle
-		      x@i <- ii
-		      x@j <- jj
-		  } else { ## i >= j : lower left triangle
-		      x@i <- jj
-		      x@j <- ii
-		  }
-	      }
-	      else { ## not any Dup
-
-		  if(x@uplo == "U") { ## i <= j : upper triangle
-		      x@i <- pmin(ii, jj)
-		      x@j <- pmax(ii, jj)
-		  } else { ## i >= j : lower left triangle
-		      x@i <- pmax(ii, jj)
-		      x@j <- pmin(ii, jj)
-		  }
-	      }
-
-	  }
-	  else if(!ip1$anyDup && !ip2$anyDup) {
-	      ## "normal case":	 no duplicated indices (and not symmetric)
-
-	      sel <- which(ip1$m  &  ip2$m)## = intersect(which(), which())
-	      x@i <- ip1$m[sel] - 1L
-	      x@j <- ip2$m[sel] - 1L
-	  }
-	  else { ## not Sym   &&  (ip1$anyDup || ip2$anyDup) :
-	      ## duplicated rows or columns -- currently the cheap solution,
-	      ## Basically  implement  X[i,j] as  X[i,] [,j] :
-	      ## FIXME: we are recomputing ip2 here
-
-	      ## - i - ------------------------------
-	      sel <- which(ip1$m > 0L)
-	      x@i <- ip1$m[sel] - 1L
-	      if(ip1$anyDup) { ## duplicated rows selected: extend sel
-		  sel <- c(sel, ip1$jm)
-		  x@i <- c(x@i, ip1$i.xtra)
-	      }
-	      x@j <- x@j[sel]
-	      if (has.x) x@x <- x@x[sel]
-
-	      ## - j - ------------------------------
-	      ## ip2 <- .ind.prep(x@j, intI(j, n = di[2], dn = dn[[2]]))
-	      ## FIXME can we do better: current x@j is original x@j[sel]
-	      ip2 <- .ind.prep(x@j, ij)
-	      sel <- which(ip2$m > 0L)
-	      x@j <- ip2$m[sel] - 1L
-	      if(ip2$anyDup) { ## duplicated columns selected: extend sel
-		  sel <- c(sel, ip2$jm)
-		  x@j <- c(x@j, ip2$i.xtra)
-	      }
-	      x@i <- x@i[sel]
-	  }
-
-	  if (has.x)
-	      x@x <- x@x[sel]
-
-	  if (drop && any(nd == 1)) drop(as(x, "matrix")) else x
-      })
-
 ## FIXME: Learn from .TM... below or rather  .M.sub.i.2col(.) in ./Matrix.R
 ## ------ the following should be much more efficient than the
 ##  subset.ij() based ./Matrix.R code :
@@ -486,7 +277,7 @@ setMethod("[", signature(x = "TsparseMatrix",
 ### -----   M[i,j] <- v  with   i,j = length-1-numeric;  v= length-1 number
 ###                            *and* M[i,j] == 0 previously
 ##
-## ---------     ----- FIXME(2): keep in sync with replCmat() in ./Csparse.R
+## FIXME(2): keep in sync with replCmat() in ./Csparse.R
 ## FIXME(3): It's terribly slow when used e.g. from diag(M[,-1]) <- value
 ## -----     which has "workhorse"   M[,-1] <- <dsparseVector>
 ##
@@ -501,15 +292,21 @@ replTmat <- function (x, i, j, ..., value)
     jMi <- missing(j)
     ## "FIXME": could pass this (and much ? more) when this function would not *be* a
     ## method but be *called* from methods
-    spV <- is(value, "sparseVector")
+
+    clDv <- getClassDef(clV <- class(value))
+    spV <- extends(clDv, "sparseVector")
     ## own version of all0() that works both for sparseVector and atomic vectors:
     .all0 <- function(v) if(spV) length(v@i) == 0 else all0(v)
-
+    delayedAssign("value.is.logical",
+                  if(spV) {
+                      extends(clDv, "lsparseVector") || extends(clDv, "nsparseVector")
+                  } else {
+                      is.logical(value) || is.logical(as.vector(value))
+                  })
     na <- nargs()
     if(na == 3) { ## i = vector (but *not* 2-col) indexing"  M[i] <- v
-	Matrix.msg(sprintf(
-		"diagnosing replTmat(x,i,j,v): nargs()= 3; missing(i,j)= (%d,%d)",
-			   iMi,jMi))
+	Matrix.msg("diagnosing replTmat(x,i,j,v): nargs()= 3; ",
+		   if(iMi | jMi) sprintf("missing (i,j) = (%d,%d)", iMi,jMi))
 	if(iMi) stop("internal bug: missing 'i' in replTmat(): please report")
 	if(is.character(i))
 	    stop("[ <character> ] indexing not allowed: forgot a \",\" ?")
@@ -525,10 +322,10 @@ replTmat <- function (x, i, j, ..., value)
 		       cl," to ",class(x))
 	}
 	nr <- di[1]
-	x.i <- .Call(m_encodeInd2, x@i, x@j, di=di)
+	x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE)
 	if(anyDuplicated(x.i)) { ## == if(is_duplicatedT(x, di = di))
 	    x <- uniqTsparse(x)
-	    x.i <- .Call(m_encodeInd2, x@i, x@j, di=di)
+	    x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE)
 	}
 
 	if(is.logical(i)) { # full-size logical indexing
@@ -542,6 +339,10 @@ replTmat <- function (x, i, j, ..., value)
         clx <- class(x)
         clDx <- getClassDef(clx) # extends(), is() etc all use the class definition
         has.x <- "x" %in% slotNames(clDx) # === slotNames(x)
+	if(!has.x && # <==> "n.TMatrix"
+	   ((iNA <- any(is.na(value))) || !value.is.logical))
+	    warning(gettextf("x[.] <- val: x is \"%s\", val not in {TRUE, FALSE} is coerced%s.",
+			     clx, if(iNA) " NA |--> TRUE" else ""))
 
 	## now have 0-based indices   x.i (entries) and	 i (new entries)
 
@@ -581,14 +382,19 @@ replTmat <- function (x, i, j, ..., value)
 	    x@factors <- list()
 	return(x)
     }
-    ## nargs() == 4 :
+    ## nargs() == 4 :  x[i,j] <- value
+    ## --------------------------------------------------------------------------
+    lenV <- length(value)
+    Matrix.msg(".. replTmat(x,i,j,v): nargs()= 4; cl.(x)=",
+	       class(x),"; len.(value)=", lenV,"; ",
+	       if(iMi | jMi) sprintf("missing (i,j) = (%d,%d)", iMi,jMi),
+	       .M.level = 2)# level 1  gives too many messages
 
     ## FIXME: use  'abIndex' or a better algorithm, e.g.  if(iMi)
     i1 <- if(iMi) 0:(di[1] - 1L) else .ind.prep2(i, 1, di, dn)
     i2 <- if(jMi) 0:(di[2] - 1L) else .ind.prep2(j, 2, di, dn)
     dind <- c(length(i1), length(i2)) # dimension of replacement region
     lenRepl <- prod(dind)
-    lenV <- length(value)
     if(lenV == 0) {
         if(lenRepl != 0)
             stop("nothing to replace with")
@@ -596,21 +402,33 @@ replTmat <- function (x, i, j, ..., value)
     }
     ## else: lenV := length(value)	 is > 0
     if(lenRepl %% lenV != 0)
-        stop("number of items to replace is not a multiple of replacement length")
-    if(!iMi && any(dup <- duplicated(i1, fromLast = TRUE))) {
-        notDup <- - which(dup)
-        i1 <- i1[notDup]
-        lenV <- length(value <- rep(value, length.out = lenRepl)[notDup])
-        dind[1] <- length(i1)
-        lenRepl <- dind[1] * dind[2]
+	stop("number of items to replace is not a multiple of replacement length")
+    if(!spV && lenRepl > 2^16) { # (somewhat arbitrary cutoff)
+	value <- as(value, "sparseVector")# so that subsequent rep(.) are fast
+        spV <- TRUE
     }
-
-    if(!jMi && any(dup <- duplicated(i2, fromLast = TRUE))) {
+    ## Now deal with duplicated / repeated indices: "last one wins"
+    if(!iMi && any(dup <- duplicated(i1, fromLast = TRUE))) { ## duplicated rows
         keep <- !dup
-        i2 <- i2[keep]
+        i1 <- i1[keep]
+        ## keep is "internally" recycled below {and that's important: it is dense!}
         lenV <- length(value <- rep(value, length.out = lenRepl)[keep])
+        dind[1] <- length(i1)
+        lenRepl <- prod(dind)
+    }
+    if(!jMi && any(dup <- duplicated(i2, fromLast = TRUE))) { ## duplicated columns
+        iDup <- which(dup)
+        ## The following is correct, but  rep(keep,..) can be *HUGE*
+        ## keep <- !dup
+        ## i2 <- i2[keep]
+        ## lenV <- length(value <- rep(value, length.out = lenRepl)[rep(keep, each=dind[1])])
+        ## solution: sv[-i] is efficient for sparseVector:
+        i2 <- i2[- iDup]
+        nr <- dind[1]
+        iDup <- rep((iDup - 1)*nr, each=nr) + seq_len(nr)
+        lenV <- length(value <- rep(value, length.out = lenRepl)[-iDup])
         dind[2] <- length(i2)
-        lenRepl <- dind[1] * dind[2]
+        lenRepl <- prod(dind)
     }
     clx <- class(x)
     clDx <- getClassDef(clx) # extends() , is() etc all use the class definition
@@ -621,17 +439,19 @@ replTmat <- function (x, i, j, ..., value)
 
     toGeneral <- r.sym <- FALSE
     if(extends(clDx, "symmetricMatrix")) {
-        mkArray <- if(spV) # TODO: room for improvement
-            function(v, dim) spV2M(v, dim[1],dim[2]) else array
-	r.sym <- (dind[1] == dind[2]) && all(i1 == i2) &&
-	(lenRepl == 1 || isSymmetric(value <- mkArray(value, dim=dind)))
+	## using array() for large dind is a disaster...
+	mkArray <- if(spV) # TODO: room for improvement
+	    function(v, dim) spV2M(v, dim[1],dim[2]) else array
+	r.sym <-
+	    (dind[1] == dind[2] && all(i1 == i2) &&
+	     (lenRepl == 1 || lenV == 1 ||
+	      isSymmetric(mkArray(value, dim=dind))))
 	if(r.sym) { ## result is *still* symmetric --> keep symmetry!
 	    xU <- x@uplo == "U"
             # later, we will consider only those indices above / below diagonal:
 	}
 	else toGeneral <- TRUE
-    }
-    else if(extends(clDx, "triangularMatrix")) {
+    } else if(extends(clDx, "triangularMatrix")) {
         xU <- x@uplo == "U"
 	r.tri <- ((any(dind == 1) || dind[1] == dind[2]) &&
 		  if(xU) max(i1) <= min(i2) else max(i2) <= min(i1))
@@ -642,7 +462,8 @@ replTmat <- function (x, i, j, ..., value)
 	else toGeneral <- TRUE
     }
     if(toGeneral) { # go to "generalMatrix" and continue
-	if((.w <- isTRUE(getOption("Matrix.warn"))) || isTRUE(getOption("Matrix.verbose")))
+	if((.w <- isTRUE(getOption("Matrix.warn"))) ||
+	   (!is.null(v <- getOption("Matrix.verbose")) && v >= 1))
 	    (if(.w) warning else message)(
 	     "M[i,j] <- v :  coercing symmetric M[] into non-symmetric")
         x <- as(x, paste(.M.kind(x), "gTMatrix", sep=''))
@@ -652,9 +473,9 @@ replTmat <- function (x, i, j, ..., value)
     ## TODO (efficiency): replace  'sel' by 'which(sel)'
     get.ind.sel <- function(ii,ij)
 	(match(x@i, ii, nomatch = 0) & match(x@j, ij, nomatch = 0))
-
     ## sel[k] := TRUE iff k-th non-zero entry (typically x@x[k]) is to be replaced
     sel <- get.ind.sel(i1,i2)
+
     has.x <- "x" %in% slotNames(clDx) # === slotNames(x)
 
     ## the simplest case: for all Tsparse, even for i or j missing
@@ -669,12 +490,16 @@ replTmat <- function (x, i, j, ..., value)
 	}
 	return(x)
     }
-
     ## else --  some( value != 0 ) --
     if(lenV > lenRepl)
         stop("too many replacement values")
     ## now have  lenV <= lenRepl
 
+    if(!has.x && # <==> "n.TMatrix"
+       ((iNA <- any(is.na(value))) || !value.is.logical))
+	warning(gettextf("x[.,.] <- val: x is \"%s\", val not in {TRUE, FALSE} is coerced%s.",
+			 clx, if(iNA) " NA |--> TRUE" else ""))
+
     ## another simple, typical case:
     if(lenRepl == 1) {
         if(spV && has.x) value <- as(value, "vector")
@@ -692,12 +517,25 @@ replTmat <- function (x, i, j, ..., value)
         return(x)
     }
 
-##     if(r.sym) # value already adjusted, see above
-##        lenRepl <- length(value) # shorter (since only "triangle")
-    if(!r.sym && lenV < lenRepl)
-        value <- rep(value, length.out = lenRepl)
+### Otherwise, for large lenRepl, we get into trouble below
 
-    ## now:  length(value) == lenRepl
+    if(lenRepl > 2^20) { # (somewhat arbitrary cutoff)
+## FIXME: just for testing !!
+## if(identical(Sys.getenv("USER"),"maechler")
+##    if(lenRepl > 2) { # __________ ___ JUST for testing! _______________
+	if(is.null(v <- getOption("Matrix.quiet")) || !v)
+	    message(gettextf("x[.,.] <- val : x being coerced from Tsparse* to CsparseMatrix"))
+	return(replCmat4(as(x,"CsparseMatrix"), i1, i2, iMi=iMi, jMi=jMi,
+			 value = if(spV) value else as(value, "sparseVector"),
+			 spV = TRUE))
+    }
+
+    ##     if(r.sym) # value already adjusted, see above
+    ##        lenRepl <- length(value) # shorter (since only "triangle")
+    if(!r.sym && lenV < lenRepl)
+	value <- rep(value, length.out = lenRepl)
+
+    ## now:  length(value) == lenRepl  {but value is sparseVector if it's "long" !}
 
     ## value[1:lenRepl]:  which are structural 0 now, which not?
     ## v0 <- is0(value)
@@ -705,17 +543,31 @@ replTmat <- function (x, i, j, ..., value)
     ## --> more efficient for sparse 'value' & large 'lenRepl' :
     ## FIXME [= FIXME(3) above]:
     ## ----- The use of  seq_len(lenRepl) below is *still* inefficient
+    ##   (or impossible e.g. when lenRepl == 50000^2)
     ##       and the  vN0 <- isN0(as.vector(value[iI0]))  is even more ...
+
+    ## One idea: use "abIndex", (a very efficient storage of index vectors which are
+    ## a concatenation of only a few arithmetic seq()ences
+    use.abI <- isTRUE(getOption("Matrix.use.abIndex"))
+    ## This 'use.abI' should later depend on the *dimension* of things !
+    ##>>> But for that, we need to implement the following abIndex - "methods":
+    ##>>>   <abI>[-n],  <value>[ <abIndex> ] , intersect(<abI>, <abI>)
+    ## and for intersect(): typically sort(), unique() & similar
+
+    iI0 <- if(use.abI) abIseq1(1L, lenRepl) else seq_len(lenRepl)
+
     if(any(sel)) {
 	## the 0-based indices of non-zero entries -- WRT to submatrix
 	non0 <- cbind(match(x@i[sel], i1),
 		      match(x@j[sel], i2)) - 1L
-	iN0 <- 1L + .Call(m_encodeInd, non0, di = dind)
+	iN0 <- 1L + .Call(m_encodeInd, non0, di = dind, FALSE)
 
 	## 1a) replace those that are already non-zero with non-0 values
-	vN0 <- isN0(as.vector(value[iN0]))
-	if(any(vN0) && has.x)
-	    x@x[sel][vN0] <- as.vector(value[iN0[vN0]])
+	vN0 <- isN0(value[iN0])
+	if(any(vN0) && has.x) {
+	    vv0 <- which(vN0)
+	    x@x[sel][vv0] <- as.vector(value[iN0[vv0]])
+	}
 
 	## 1b) replace non-zeros with 0 --> drop entries
 	if(any(!vN0)) {
@@ -725,35 +577,45 @@ replTmat <- function (x, i, j, ..., value)
 	    x@i <- x@i[-ii]
 	    x@j <- x@j[-ii]
 	}
-	iI0 <- if(length(iN0) < lenRepl)
-	    seq_len(lenRepl)[-iN0] # == complementInd(non0, dind)
-    } else iI0 <- seq_len(lenRepl)
-
+	iI0 <- if(length(iN0) < lenRepl) iI0[-iN0] ## else NULL
+                                        # == complementInd(non0, dind)
+    }
     if(length(iI0)) {
         if(r.sym) {
-	    ## should only set new entries above / below diagonal
-	    iSel <- indTri(dind[1], upper=xU, diag=TRUE)
+	    ## should only set new entries above / below diagonal, i.e.,
+            ## subset iI0 such as to contain only  above/below ..
+	    iSel <-
+		if(use.abI) abIindTri(dind[1], upper=xU, diag=TRUE)
+		else	       indTri(dind[1], upper=xU, diag=TRUE)
 	    ## select also the corresponding triangle of values
+### TODO for "abIndex" -- note we KNOW that both  iI0 and iSel
+### are strictly increasing :
 	    iI0 <- intersect(iI0, iSel)
         }
-        if(any(vN0 <- isN0(as.vector(value[iI0])))) {
-            ## 2) add those that were structural 0 (where value != 0)
-            ij0 <- decodeInd(iI0[vN0] - 1L, nr = dind[1])
-            x@i <- c(x@i, i1[ij0[,1] + 1L])
-            x@j <- c(x@j, i2[ij0[,2] + 1L])
-            if(has.x)
-		x@x <- c(x@x, as.vector(value[iI0[vN0]]))
-        }
+        full <- length(iI0) == lenRepl
+	vN0 <-
+	    if(spV) ## "sparseVector"
+		(if(full) value else value[iI0])@i
+	    else which(isN0(if(full) value else value[iI0]))
+	if(length(vN0)) {
+	    ## 2) add those that were structural 0 (where value != 0)
+	    iIN0 <- if(full) vN0 else iI0[vN0]
+	    ij0 <- decodeInd(iIN0 - 1L, nr = dind[1])
+	    x@i <- c(x@i, i1[ij0[,1] + 1L])
+	    x@j <- c(x@j, i2[ij0[,2] + 1L])
+	    if(has.x)
+		x@x <- c(x@x, as.vector(value[iIN0]))
+	}
     }
     if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
 	x@factors <- list()
     x
 } ## end{replTmat}
 
-## A[ ij ] <- value,  where ij is (i,j) 2-column matrix :
+## A[ ij ] <- value,  where ij is a matrix; typically (i,j) 2-column matrix :
 ## ----------------   ./Matrix.R has a general cheap method
 ## This one should become as fast as possible -- is also used from Csparse.R --
-.TM.repl.i.2col <- function (x, i, j, ..., value)
+.TM.repl.i.mat <- function (x, i, j, ..., value)
 {
     nA <- nargs()
     if(nA != 3)
@@ -761,10 +623,15 @@ replTmat <- function (x, i, j, ..., value)
 
     ## else: nA == 3  i.e.,  M [ cbind(ii,jj) ] <- value or M [ Lmat ] <- value
     if(is.logical(i)) {
-	Matrix.msg(".TM.repl.i.2col(): drop 'matrix' case ...")
+	Matrix.msg(".TM.repl.i.mat(): drop 'matrix' case ...", .M.level=2)
 	## c(i) : drop "matrix" to logical vector
-	x[c(i)] <- value
+	x[as.vector(i)] <- value
 	return(x)
+    } else if(extends(cli <- getClassDef(class(i)),"lMatrix") || extends(cli, "nMatrix")) {
+	Matrix.msg(".TM.repl.i.mat(): \"lMatrix\" case ...", .M.level=2)
+	i <- which(as(i, if(extends(cli, "sparseMatrix")) "sparseVector" else "vector"))
+	## x[i] <- value ; return(x)
+	return(`[<-`(x,i, value=value))
     } else if(!is.numeric(i) || ncol(i) != 2)
 	stop("such indexing must be by logical or 2-column numeric matrix")
     if(!is.integer(i)) storage.mode(i) <- "integer"
@@ -847,7 +714,7 @@ replTmat <- function (x, i, j, ..., value)
 	clDx <- getClassDef(clx <- class(x))
     }
 
-    ii.v <- .Call(m_encodeInd, i - 1L, di)# 0-indexing
+    ii.v <- .Call(m_encodeInd, i - 1L, di, checkBounds = TRUE)# 0-indexing
     if(any(d <- duplicated(rev(ii.v)))) { # reverse: "last" duplicated FALSE
 	warning("duplicate ij-entries in 'Matrix[ ij ] <- value'; using last")
 	nd <- !rev(d)
@@ -855,7 +722,7 @@ replTmat <- function (x, i, j, ..., value)
 	ii.v  <- ii.v [nd]
 	value <- value[nd]
     }
-    ii.x <- .Call(m_encodeInd2, x@i, x@j, di)
+    ii.x <- .Call(m_encodeInd2, x@i, x@j, di, FALSE)
     m1 <- match(ii.v, ii.x)
     i.repl <- !is.na(m1) # those that need to be *replaced*
 
@@ -878,18 +745,18 @@ replTmat <- function (x, i, j, ..., value)
 	    }
 	}
     }
-    if(!all(i.repl)) { ## some new entries
-	i.j <- decodeInd(ii.v[!i.repl], nr)
+    if(any(i.new <- !i.repl & isN0(value))) { ## some new entries
+	i.j <- decodeInd(ii.v[i.new], nr)
 	x@i <- c(x@i, i.j[,1])
 	x@j <- c(x@j, i.j[,2])
 	if(has.x)
-	    x@x <- c(x@x, value[!i.repl])
+	    x@x <- c(x@x, value[i.new])
     }
 
     if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
 	x@factors <- list()
     x
-} ## end{.TM.repl.i.2col}
+} ## end{.TM.repl.i.mat}
 
 setReplaceMethod("[", signature(x = "TsparseMatrix", i = "index", j = "missing",
 				value = "replValue"),
@@ -905,7 +772,7 @@ setReplaceMethod("[", signature(x = "TsparseMatrix", i = "index", j = "index",
 
 setReplaceMethod("[", signature(x = "TsparseMatrix", i = "matrix", j = "missing",
 				value = "replValue"),
-		 .TM.repl.i.2col)
+		 .TM.repl.i.mat)
 
 
 ### When the RHS 'value' is  a sparseVector, now can use  replTmat  as well
