@@ -1,33 +1,10 @@
 ###      Methods for generics from newGenerics.q for some standard classes
 ###
-### Copyright 1997-2003  Jose C. Pinheiro <Jose.Pinheiro@pharma.novartis.com>,
+### Copyright 1997-2003  Jose C. Pinheiro,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
 
 ##*## Methods for some of the generics in newGenerics.q for standard classes
 
-BIC.logLik <-
-  ## BIC for logLik objects
-  function(object, ...)
-{
-  -2 * (c(object) - attr(object, "df") * log(attr(object, "nobs"))/2)
-}
-
-BIC.lm <- BIC.nls <-
-  ## BIC for various fitted objects
-  function(object, ...)
-{
-  if(nargs() > 1) {
-    object <- list(object, ...)
-    val <- lapply(object, logLik)
-    val <-
-      as.data.frame(t(sapply(val, function(el) c(attr(el, "df"), BIC(el)))))
-    names(val) <- c("df", "BIC")
-    row.names(val) <- as.character(match.call()[-1])
-    return(val)
-  }
-  BIC(logLik(object))
-}
-
 Dim.default <- function(object, ...) dim(object)
 
 getCovariate.data.frame <-
@@ -52,9 +29,12 @@ getData.nls <-
   ## avoid partial matches here.
   data <- eval(if("data" %in% names(object)) object$data else mCall$data)
   if (is.null(data)) return(data)
-  naAct <- eval(mCall$na.action)
+  naAct <- object[["na.action"]]
   if (!is.null(naAct)) {
-    data <- naAct(data)
+      ## guessing here: known values (omit, exclude) work.
+      data <- if (inherits(naAct, "omit")) data[-naAct, ]
+      else if (inherits(naAct, "exclude")) data
+      else eval(mCall$na.action)(data)
   }
   subset <- mCall$subset
   if (!is.null(subset)) {
