@@ -26,16 +26,8 @@
 #include "tre-stack.h"
 #include "tre-parse.h"
 
-/* fake definition */
-extern void Rf_error(const char *str);
 #define assert(a) R_assert(a)
 
-static void assert(int expr)
-{
-    if(expr == 0)
-	Rf_error("internal error in TRE");
-}
-
 /* Characters with special meanings in regexp syntax. */
 #define CHAR_PIPE	   L'|'
 #define CHAR_LPAREN	   L'('
