@@ -209,6 +209,7 @@ format.POSIXct <- function(x, format = "", tz = "", usetz = FALSE, ...)
               names=names(x))
 }
 
+## could handle arrays for max.print
 print.POSIXct <- function(x, ...)
 {
     max.print <- getOption("max.print", 9999L)
@@ -891,7 +892,7 @@ round.POSIXt <- function(x, units=c("secs", "mins", "hours", "days"))
 
 `[.POSIXlt` <- function(x, ..., drop = TRUE)
 {
-    val <- lapply(x, "[", ..., drop = drop)
+    val <- lapply(X = x, FUN = "[", ..., drop = drop)
     attributes(val) <- attributes(x) # need to preserve timezones
     val
 }
@@ -925,7 +926,7 @@ rep.POSIXct <- function(x, ...)
 
 rep.POSIXlt <- function(x, ...)
 {
-    y <- lapply(x, rep, ...)
+    y <- lapply(X = x, FUN = rep, ...)
     attributes(y) <- attributes(x)
     y
 }
