@@ -27,10 +27,15 @@ parse <- function(file = "", n = NULL, text = NULL, prompt = "?",
     if(is.character(file))
         if(file == "") file <- stdin()
         else {
-            if (missing(srcfile) && keep.source)
-        	srcfile <- srcfile(file, Enc = encoding)
-            file <- file(file, "r")
-            on.exit(close(file))
+            filename <- file
+            file <- file(filename, "r")
+            if (missing(srcfile) && keep.source) {
+            	text <- readLines(file)
+            	close(file)
+            	file <- stdin()
+        	srcfile <- srcfilecopy(filename, text)
+            } else 	
+                on.exit(close(file))
         }
     .Internal(parse(file, n, text, prompt, srcfile, encoding))
 }
