@@ -26,8 +26,8 @@ function(x, y = NULL, labels = seq_along(x),
     }
     labels <- as.graphicsAnnot(labels)
     if (!is.null(vfont))
-        vfont <- c(typeface = pmatch(vfont[1], Hershey$typeface),
-                   fontindex= pmatch(vfont[2], Hershey$fontindex))
+        vfont <- c(typeface = pmatch(vfont[1L], Hershey$typeface),
+                   fontindex= pmatch(vfont[2L], Hershey$fontindex))
     .Internal(text(xy.coords(x,y, recycle = TRUE), labels,
                    adj, pos, offset, vfont, cex, col, font, ...))
 }
