@@ -82,12 +82,12 @@ function (x,
     ## Call to filtering loop
     len <- length(x) - start.time + 1
     hw <- function(alpha, beta, gamma)
-        .C("HoltWinters",
+        .C(C_HoltWinters,
            as.double(x),
            as.integer(length(x)),
-           as.double(alpha),
-           as.double(beta),
-           as.double(gamma),
+           as.double(max(min(alpha,1),0)),
+           as.double(max(min(beta,1),0)),
+           as.double(max(min(gamma,1),0)),
            as.integer(start.time),
            as.integer(! + (seasonal == "multiplicative")),
            as.integer(f),
@@ -120,6 +120,8 @@ function (x,
                 sol   <- optim(optim.start, error, method = "L-BFGS-B",
                                lower = c(0, 0, 0), upper = c(1, 1, 1),
                                control = optim.control)
+                if(sol$convergence || any(sol$par < 0 | sol$par > 1))
+                    stop("optimization failure")
                 alpha <- sol$par[1L]
                 beta  <- sol$par[2L]
                 gamma <- sol$par[3L]
@@ -131,6 +133,8 @@ function (x,
                                error, method = "L-BFGS-B",
                                lower = c(0, 0), upper = c(1, 1),
                                control = optim.control)
+                if(sol$convergence || any(sol$par < 0 | sol$par > 1))
+                    stop("optimization failure")
                 alpha <- sol$par[1L]
                 gamma <- sol$par[2L]
             }
@@ -144,6 +148,8 @@ function (x,
                                error, method = "L-BFGS-B",
                                lower = c(0, 0), upper = c(1, 1),
                                control = optim.control)
+                if(sol$convergence || any(sol$par < 0 | sol$par > 1))
+                    stop("optimization failure")
                 beta  <- sol$par[1L]
                 gamma <- sol$par[2L]
             } else {
@@ -165,6 +171,8 @@ function (x,
                                error, method = "L-BFGS-B",
                                lower = c(0, 0), upper = c(1, 1),
                                control = optim.control)
+                if(sol$convergence || any(sol$par < 0 | sol$par > 1))
+                    stop("optimization failure")
                 alpha <- sol$par[1L]
                 beta  <- sol$par[2L]
             } else {
@@ -280,6 +288,7 @@ plot.HoltWinters <-
 
     preds <- length(predicted.values) > 1 || !is.na(predicted.values)
 
+    dev.hold(); on.exit(dev.flush())
     ## plot fitted/predicted values
     plot(ts(c(fitted(x)[,1], if(preds) predicted.values[,1]),
             start = start(fitted(x)[,1]),
@@ -364,7 +373,8 @@ function (x, type = c("additive", "multiplicative"), filter = NULL)
                    start = start(x), frequency = f)
 
     ## return values
-    structure(list(seasonal = seasonal,
+    structure(list(x = x,
+                   seasonal = seasonal,
                    trend = trend,
                    random = if (type == "additive")
                        x - seasonal - trend
@@ -377,11 +387,11 @@ function (x, type = c("additive", "multiplicative"), filter = NULL)
 
 plot.decomposed.ts <- function(x, ...)
 {
-    plot(cbind(
-               observed = if (x$type == "additive")
-                 x$random + x$trend + x$seasonal
-               else
-                 x$random * x$trend * x$seasonal,
+    xx <- x$x # added in 2.14.0
+    if(is.null(xx))
+        xx <- with(x,  if (type == "additive") random + trend + seasonal
+                       else random * trend * seasonal)
+    plot(cbind(observed = xx,
                trend    = x$trend,
                seasonal = x$seasonal,
                random   = x$random
