@@ -27,6 +27,8 @@ safe_pf <- function(q, df1, ...)
     pf(q=q, df1=df1, ...)
 }
 
+## NB: functions in this file will use the 'stats' S3 generics for
+## nobs(), terms() ....
 
 add1 <- function(object, scope, ...) UseMethod("add1")
 
@@ -46,7 +48,7 @@ add1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
     ans <- matrix(nrow = ns + 1L, ncol = 2L,
                   dimnames = list(c("<none>", scope), c("df", "AIC")))
     ans[1L,  ] <- extractAIC(object, scale, k = k, ...)
-    n0 <- length(object$residuals)
+    n0 <- nobs(object, use.fallback = TRUE)
     env <- environment(formula(object))
     for(i in seq(ns)) {
 	tt <- scope[i]
@@ -58,7 +60,8 @@ add1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
                        evaluate = FALSE)
 	nfit <- eval(nfit, envir=env) # was  eval.parent(nfit)
 	ans[i+1L, ] <- extractAIC(nfit, scale, k = k, ...)
-        if(length(nfit$residuals) != n0)
+        nnew <- nobs(nfit, use.fallback = TRUE)
+        if(all(is.finite(c(n0, nnew))) && nnew != n0)
             stop("number of rows in use has changed: remove missing values?")
     }
     dfs <- ans[, 1L] - ans[1L, 1L]
@@ -71,16 +74,18 @@ add1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
 	nas <- !is.na(dev)
 	P <- dev
 	P[nas] <- safe_pchisq(dev[nas], dfs[nas], lower.tail=FALSE)
-	aod[, c("LRT", "Pr(Chi)")] <- list(dev, P)
+	aod[, c("LRT", "Pr(>Chi)")] <- list(dev, P)
     }
-    head <- c("Single term additions", "\nModel:",
-	      deparse(as.vector(formula(object))),
+    head <- c("Single term additions", "\nModel:", deparse(formula(object)),
 	      if(scale > 0) paste("\nscale: ", format(scale), "\n"))
     class(aod) <- c("anova", "data.frame")
     attr(aod, "heading") <- head
     aod
 }
 
+##' @title Check for exact fit
+##' @param object an lm object (hence using "$" instead of methods)
+##' @return (unused / nothing explicitly)
 check_exact <- function(object)
 {
     w <- object$weights
@@ -92,7 +97,8 @@ check_exact <- function(object)
         rss <- sum(w * object$residuals^2)
     }
     if(rss < 1e-10*mss)
-        warning("attempting model selection on an essentially perfect fit is nonsense", call. = FALSE)
+	warning("attempting model selection on an essentially perfect fit is nonsense",
+		call. = FALSE)
 }
 
 add1.lm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
@@ -193,20 +199,20 @@ add1.lm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
         df <- aod$Df
         nas <- !is.na(df)
         dev[nas] <- safe_pchisq(dev[nas], df[nas], lower.tail=FALSE)
-        aod[, "Pr(Chi)"] <- dev
+        aod[, "Pr(>Chi)"] <- dev
     } else if(test == "F") {
 	rdf <- object$df.residual
-	aod[, c("F value", "Pr(F)")] <- Fstat(aod, aod$RSS[1L], rdf)
+	aod[, c("F value", "Pr(>F)")] <- Fstat(aod, aod$RSS[1L], rdf)
     }
-    head <- c("Single term additions", "\nModel:",
-	      deparse(as.vector(formula(object))),
+    head <- c("Single term additions", "\nModel:", deparse(formula(object)),
 	      if(scale > 0) paste("\nscale: ", format(scale), "\n"))
     class(aod) <- c("anova", "data.frame")
     attr(aod, "heading") <- head
     aod
 }
 
-add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
+add1.glm <- function(object, scope, scale = 0, test=c("none", "Rao", "LRT",
+                                                 "Chisq", "F"),
 		     x = NULL, k = 2, ...)
 {
     Fstat <- function(table, rdf) {
@@ -220,6 +226,9 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 	P[nnas] <- safe_pf(Fs[nnas], df[nnas], rdf - df[nnas], lower.tail=FALSE)
 	list(Fs=Fs, P=P)
     }
+    test <- match.arg(test)
+    if (test=="Chisq") test <- "LRT"
+
     if(!is.character(scope))
 	scope <- add.scope(object, update.formula(object, scope))
     if(!length(scope))
@@ -227,8 +236,8 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     oTerms <- attr(object$terms, "term.labels")
     int <- attr(object$terms, "intercept")
     ns <- length(scope)
-    dfs <- dev <- numeric(ns+1)
-    names(dfs) <- names(dev) <- c("<none>", scope)
+    dfs <- dev <- score <- numeric(ns+1)
+    names(dfs) <- names(dev) <- names(score) <- c("<none>", scope)
     add.rhs <- paste(scope, collapse = "+")
     add.rhs <- eval(parse(text = paste("~ . +", add.rhs)))
     new.form <- update.formula(object, add.rhs)
@@ -274,6 +283,8 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
                   family=object$family, control=object$control)
     dfs[1L] <- z$rank
     dev[1L] <- z$deviance
+    r <- z$residuals
+    w <- z$weights
     ## workaround for PR#7842. terms.formula may have flipped interactions
     sTerms <- sapply(strsplit(Terms, ":", fixed=TRUE),
                      function(x) paste(sort(x), collapse=":"))
@@ -285,6 +296,11 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 		      family=object$family, control=object$control)
 	dfs[tt] <- z$rank
 	dev[tt] <- z$deviance
+        if (test=="Rao") {
+          ## WLS for score test (comes out as model SS)
+          zz <- glm.fit(X, r, w, offset=offset)
+          score[tt] <- zz$null.deviance - zz$deviance
+        }
     }
     if (scale == 0)
 	dispersion <- summary(object, dispersion = NULL)$dispersion
@@ -302,23 +318,31 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 		      row.names = names(dfs), check.names = FALSE)
     if(all(is.na(aic))) aod <- aod[, -3]
     test <- match.arg(test)
-    if(test == "Chisq") {
+    if(test == "LRT") {
         dev <- pmax(0, loglik[1L] - loglik)
         dev[1L] <- NA
         LRT <- if(dispersion == 1) "LRT" else "scaled dev."
         aod[, LRT] <- dev
         nas <- !is.na(dev)
         dev[nas] <- safe_pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
-        aod[, "Pr(Chi)"] <- dev
+        aod[, "Pr(>Chi)"] <- dev
+    } else if(test == "Rao") {
+        dev <- pmax(0, score) # roundoff guard
+        dev[1L] <- NA
+        nas <- !is.na(dev)
+        SC <- if(dispersion == 1) "Rao score" else "scaled Rao sc."
+        dev <- dev/dispersion
+        aod[, SC] <- dev
+        dev[nas] <- safe_pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
+        aod[, "Pr(>Chi)"] <- dev
     } else if(test == "F") {
         if(fam == "binomial" || fam == "poisson")
             warning(gettextf("F test assumes quasi%s family", fam),
                     domain = NA)
 	rdf <- object$df.residual
-	aod[, c("F value", "Pr(F)")] <- Fstat(aod, rdf)
+	aod[, c("F value", "Pr(>F)")] <- Fstat(aod, rdf)
     }
-    head <- c("Single term additions", "\nModel:",
-	      deparse(as.vector(formula(object))),
+    head <- c("Single term additions", "\nModel:", deparse(formula(object)),
 	      if(scale > 0) paste("\nscale: ", format(scale), "\n"))
     class(aod) <- c("anova", "data.frame")
     attr(aod, "heading") <- head
@@ -333,7 +357,7 @@ drop1 <- function(object, scope, ...) UseMethod("drop1")
 drop1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
 			  k = 2, trace = FALSE, ...)
 {
-    tl <- attr(object$terms, "term.labels")
+    tl <- attr(terms(object), "term.labels")
     if(missing(scope)) scope <- drop.scope(object)
     else {
 	if(!is.character(scope))
@@ -341,13 +365,11 @@ drop1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
 	if(!all(match(scope, tl, 0L) > 0L))
 	    stop("scope is not a subset of term labels")
     }
-#    data <- model.frame(object) # remove NAs
-#    object <- update(object, data = data)
     ns <- length(scope)
     ans <- matrix(nrow = ns + 1L, ncol = 2L,
                   dimnames =  list(c("<none>", scope), c("df", "AIC")))
     ans[1, ] <- extractAIC(object, scale, k = k, ...)
-    n0 <- length(object$residuals)
+    n0 <- nobs(object, use.fallback = TRUE)
     env <- environment(formula(object))
     for(i in seq(ns)) {
 	tt <- scope[i]
@@ -359,7 +381,8 @@ drop1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
                        evaluate = FALSE)
 	nfit <- eval(nfit, envir=env) # was  eval.parent(nfit)
 	ans[i+1, ] <- extractAIC(nfit, scale, k = k, ...)
-        if(length(nfit$residuals) != n0)
+        nnew <- nobs(nfit, use.fallback = TRUE)
+        if(all(is.finite(c(n0, nnew))) && nnew != n0)
             stop("number of rows in use has changed: remove missing values?")
     }
     dfs <- ans[1L , 1L] - ans[, 1L]
@@ -372,10 +395,9 @@ drop1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
         nas <- !is.na(dev)
         P <- dev
         P[nas] <- safe_pchisq(dev[nas], dfs[nas], lower.tail = FALSE)
-        aod[, c("LRT", "Pr(Chi)")] <- list(dev, P)
+        aod[, c("LRT", "Pr(>Chi)")] <- list(dev, P)
     }
-    head <- c("Single term deletions", "\nModel:",
-	      deparse(as.vector(formula(object))),
+    head <- c("Single term deletions", "\nModel:", deparse(formula(object)),
 	      if(scale > 0) paste("\nscale: ", format(scale), "\n"))
     class(aod) <- c("anova", "data.frame")
     attr(aod, "heading") <- head
@@ -439,7 +461,7 @@ drop1.lm <- function(object, scope, scale = 0, all.cols = TRUE,
         df <- aod$Df
         nas <- !is.na(df)
         dev[nas] <- safe_pchisq(dev[nas], df[nas], lower.tail=FALSE)
-        aod[, "Pr(Chi)"] <- dev
+        aod[, "Pr(>Chi)"] <- dev
     } else if(test == "F") {
 	dev <- aod$"Sum of Sq"
 	dfs <- aod$Df
@@ -450,10 +472,9 @@ drop1.lm <- function(object, scope, scale = 0, all.cols = TRUE,
 	P <- Fs
 	nas <- !is.na(Fs)
 	P[nas] <- safe_pf(Fs[nas], dfs[nas], rdf, lower.tail=FALSE)
-	aod[, c("F value", "Pr(F)")] <- list(Fs, P)
+	aod[, c("F value", "Pr(>F)")] <- list(Fs, P)
     }
-    head <- c("Single term deletions", "\nModel:",
-	      deparse(as.vector(formula(object))),
+    head <- c("Single term deletions", "\nModel:", deparse(formula(object)),
 	      if(scale > 0) paste("\nscale: ", format(scale), "\n"))
     class(aod) <- c("anova", "data.frame")
     attr(aod, "heading") <- head
@@ -463,9 +484,11 @@ drop1.lm <- function(object, scope, scale = 0, all.cols = TRUE,
 drop1.mlm <- function(object, scope, ...)
     stop("no 'drop1' method for \"mlm\" models")
 
-drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
+drop1.glm <- function(object, scope, scale = 0, test=c("none", "Rao", "LRT", "Chisq", "F"),
 		      k = 2, ...)
 {
+    test <- match.arg(test)
+    if (test=="Chisq") test <- "LRT"
     x <- model.matrix(object)
 #    iswt <- !is.null(wt <- object$weights)
     n <- nrow(x)
@@ -484,6 +507,7 @@ drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     chisq <- object$deviance
     dfs <- numeric(ns)
     dev <- numeric(ns)
+    score <- numeric(ns)
     y <- object$y
     if(is.null(y)) {
         y <- model.response(model.frame(object))
@@ -499,10 +523,22 @@ drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 		      family=object$family, control=object$control)
 	dfs[i] <- z$rank
 	dev[i] <- z$deviance
+
+        if (test=="Rao"){
+            r <- z$residuals
+            w <- z$weights
+            ## Approximative refit of full model to residuals using WLS
+            ## Score statistic comes out as (weighted) model SS
+            zz <- glm.fit(x, r, w, offset=object$offset)
+            score[i] <- zz$null.deviance - zz$deviance
+        }
     }
     scope <- c("<none>", scope)
     dfs <- c(object$rank, dfs)
     dev <- c(chisq, dev)
+    if (test=="Rao") {
+      score <- c(NA, score)
+    }
     dispersion <- if (is.null(scale) || scale == 0)
 	summary(object, dispersion = NULL)$dispersion
     else scale
@@ -518,15 +554,22 @@ drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     aod <- data.frame(Df = dfs, Deviance = dev, AIC = aic,
 		      row.names = scope, check.names = FALSE)
     if(all(is.na(aic))) aod <- aod[, -3]
-    test <- match.arg(test)
-    if(test == "Chisq") {
+    if(test == "LRT") {
         dev <- pmax(0, loglik - loglik[1L])
         dev[1L] <- NA
         nas <- !is.na(dev)
         LRT <- if(dispersion == 1) "LRT" else "scaled dev."
         aod[, LRT] <- dev
         dev[nas] <- safe_pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
-        aod[, "Pr(Chi)"] <- dev
+        aod[, "Pr(>Chi)"] <- dev
+    } else if(test == "Rao") {
+        dev <- pmax(0, score) # roundoff guard
+        nas <- !is.na(dev)
+        SC <- if(dispersion == 1) "Rao score" else "scaled Rao sc."
+        dev <- dev/dispersion
+        aod[, SC] <- dev
+        dev[nas] <- safe_pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
+        aod[, "Pr(>Chi)"] <- dev
     } else if(test == "F") {
         if(fam == "binomial" || fam == "poisson")
             warning(gettextf("F test assumes 'quasi%s' family", fam),
@@ -541,10 +584,9 @@ drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 	P <- Fs
 	nas <- !is.na(Fs)
 	P[nas] <- safe_pf(Fs[nas], dfs[nas], rdf, lower.tail=FALSE)
-	aod[, c("F value", "Pr(F)")] <- list(Fs, P)
+	aod[, c("F value", "Pr(>F)")] <- list(Fs, P)
     }
-    head <- c("Single term deletions", "\nModel:",
-	      deparse(as.vector(formula(object))),
+    head <- c("Single term deletions", "\nModel:", deparse(formula(object)),
 	      if(!is.null(scale) && scale > 0)
 	      paste("\nscale: ", format(scale), "\n"))
     class(aod) <- c("anova", "data.frame")
@@ -563,7 +605,7 @@ add.scope <- function(terms1, terms2)
 drop.scope <- function(terms1, terms2)
 {
     terms1 <- terms(terms1)
-    f2 <- if(missing(terms2)) numeric(0L)
+    f2 <- if(missing(terms2)) numeric()
     else attr(terms(terms2), "factors")
     factor.scope(attr(terms1, "factors"), list(drop = f2))$drop
 }
@@ -599,9 +641,9 @@ factor.scope <- function(factor, scope)
 	    for(i in seq(keep)) keep[i] <- max(f[i, - i]) != f[i, i]
 	    nmdrop <- nmdrop[keep]
 	}
-    } else nmdrop <- character(0L)
+    } else nmdrop <- character()
 
-    if(!length(add)) nmadd <- character(0L)
+    if(!length(add)) nmadd <- character()
     else {
 	nmfac <- colnames(factor)
 	nmadd <- colnames(add)
@@ -682,8 +724,8 @@ step <- function(object, scope, scale = 0,
 	ddf <- c(NA, diff(rdf))
 	AIC <- sapply(models, "[[", "AIC")
 	heading <- c("Stepwise Model Path \nAnalysis of Deviance Table",
-		     "\nInitial Model:", deparse(as.vector(formula(object))),
-		     "\nFinal Model:", deparse(as.vector(formula(fit))),
+		     "\nInitial Model:", deparse(formula(object)),
+		     "\nFinal Model:", deparse(formula(fit)),
 		     "\n")
 	aod <- data.frame(Step = I(change), Df = ddf, Deviance = dd,
                           "Resid. Df" = rdf, "Resid. Dev" = rd, AIC = AIC,
@@ -706,7 +748,7 @@ step <- function(object, scope, scale = 0,
     backward <- direction == "both" | direction == "backward"
     forward  <- direction == "both" | direction == "forward"
     if(missing(scope)) {
-	fdrop <- numeric(0L)
+	fdrop <- numeric()
         fadd <- attr(Terms, "factors")
         if(md) forward <- FALSE
     }
@@ -714,19 +756,19 @@ step <- function(object, scope, scale = 0,
 	if(is.list(scope)) {
 	    fdrop <- if(!is.null(fdrop <- scope$lower))
 		attr(terms(update.formula(object, fdrop)), "factors")
-	    else numeric(0L)
+	    else numeric()
 	    fadd <- if(!is.null(fadd <- scope$upper))
 		attr(terms(update.formula(object, fadd)), "factors")
 	}
         else {
 	    fadd <- if(!is.null(fadd <- scope))
 		attr(terms(update.formula(object, scope)), "factors")
-	    fdrop <- numeric(0L)
+	    fdrop <- numeric()
 	}
     }
     models <- vector("list", steps)
     if(!is.null(keep)) keep.list <- vector("list", steps)
-    n <- length(object$residuals)
+    n <- nobs(object, use.fallback = TRUE)  # might be NA
     fit <- object
     bAIC <- extractAIC(fit, scale, k = k, ...)
     edf <- bAIC[1L]
@@ -734,11 +776,14 @@ step <- function(object, scope, scale = 0,
     if(is.na(bAIC))
         stop("AIC is not defined for this model, so 'step' cannot proceed")
     nm <- 1
-    Terms <- fit$terms
-    if(trace)
+    ## Terms <- fit$terms
+    if(trace) {
 	cat("Start:  AIC=", format(round(bAIC, 2)), "\n",
-	    cut.string(deparse(as.vector(formula(fit)))), "\n\n", sep='')
+	    cut.string(deparse(formula(fit))), "\n\n", sep='')
+        utils::flush.console()
+    }
 
+    ## FIXME think about df.residual() here
     models[[nm]] <- list(deviance = mydeviance(fit), df.resid = n - edf,
 			 change = "", AIC = bAIC)
     if(!is.null(keep)) keep.list[[nm]] <- keep(fit, bAIC)
@@ -789,18 +834,22 @@ step <- function(object, scope, scale = 0,
         ## may need to look for a `data' argument in parent
 	fit <- update(fit, paste("~ .", change), evaluate = FALSE)
         fit <- eval.parent(fit)
-        if(length(fit$residuals) != n)
+        nnew <- nobs(fit, use.fallback = TRUE)
+        if(all(is.finite(c(n, nnew))) && nnew != n)
             stop("number of rows in use has changed: remove missing values?")
         Terms <- terms(fit)
 	bAIC <- extractAIC(fit, scale, k = k, ...)
 	edf <- bAIC[1L]
 	bAIC <- bAIC[2L]
-	if(trace)
+	if(trace) {
 	    cat("\nStep:  AIC=", format(round(bAIC, 2)), "\n",
-		cut.string(deparse(as.vector(formula(fit)))), "\n\n", sep='')
+		cut.string(deparse(formula(fit))), "\n\n", sep='')
+            utils::flush.console()
+        }
         ## add a tolerance as dropping 0-df terms might increase AIC slightly
 	if(bAIC >= AIC + 1e-7) break
 	nm <- nm + 1
+        ## FIXME: think about using df.residual() here.
 	models[[nm]] <-
 	    list(deviance = mydeviance(fit), df.resid = n - edf,
 		 change = change, AIC = bAIC)
@@ -831,7 +880,7 @@ extractAIC.survreg <- function(fit, scale, k = 2, ...)
 extractAIC.glm <- function(fit, scale = 0, k = 2, ...)
 {
     n <- length(fit$residuals)
-    edf <- n  - fit$df.residual
+    edf <- n  - fit$df.residual # assumes dispersion is known
     aic <- fit$aic
     c(edf, aic + (k-2) * edf)
 }
@@ -839,7 +888,7 @@ extractAIC.glm <- function(fit, scale = 0, k = 2, ...)
 extractAIC.lm <- function(fit, scale = 0, k = 2, ...)
 {
     n <- length(fit$residuals)
-    edf <- n  - fit$df.residual
+    edf <- n  - fit$df.residual # maybe -1 if sigma^2 is estimated
     RSS <- deviance.lm(fit)
     dev <- if(scale > 0) RSS/scale - n else n * log(RSS/n)
     c(edf, dev + k * edf)
@@ -849,6 +898,6 @@ extractAIC.aov <- extractAIC.lm
 extractAIC.negbin <- function(fit, scale, k = 2, ...)
 {
     n <- length(fit$residuals)
-    edf <- n - fit$df.residual
+    edf <- n - fit$df.residual # may -1 if theta is estimated
     c(edf, -fit$twologlik + k * edf)
 }
