@@ -40,7 +40,7 @@ print.ecdf <- function (x, digits= getOption("digits") - 2, ...)
     print(attr(x, "call"), ...)
     n <- length(xx <- environment(x)$"x")
     i1 <- 1L:min(3L,n)
-    i2 <- if(n >= 4L) max(4L,n-1L):n else integer(0L)
+    i2 <- if(n >= 4L) max(4L, n-1L):n else integer()
     cat(" x[1:",n,"] = ", numform(xx[i1]),
 	if(n>3L) ", ", if(n>5L) " ..., ", numform(xx[i2]), "\n", sep = "")
     invisible(x)
@@ -76,5 +76,5 @@ plot.ecdf <- function(x, ..., ylab="Fn(x)", verticals = FALSE,
 
 quantile.ecdf <- function (x, ...)
     ## == quantile( sort( <original sample> ) ) :
-    quantile(eval(rep.int(x, diff(c(0,round(nobs*y)))), environment(x)), ...)
+    quantile(evalq(rep.int(x, diff(c(0,round(nobs*y)))), environment(x)), ...)
 
