@@ -19,15 +19,15 @@ ftable <- function(x, ...) UseMethod("ftable")
 ftable.default <- function(..., exclude = c(NA, NaN),
                            row.vars = NULL, col.vars = NULL) {
     args <- list(...)
-    if (length(args) == 0)
+    if (length(args) == 0L)
         stop("nothing to tabulate")
-    x <- args[[1]]
+    x <- args[[1L]]
     if(is.list(x))
         x <- table(x, exclude = exclude)
     else if(inherits(x, "ftable")) {
         x <- as.table(x)
     }
-    else if(!(is.array(x) && (length(dim(x)) > 1))) {
+    else if(!(is.array(x) && (length(dim(x)) > 1L))) {
         x <- table(..., exclude = exclude)
     }
     dn <- dimnames(x)
@@ -83,7 +83,7 @@ ftable.formula <- function(formula, data = NULL, subset, na.action, ...)
 {
     if(missing(formula) || !inherits(formula, "formula"))
         stop("'formula' missing or incorrect")
-    if(length(formula) != 3)
+    if(length(formula) != 3L)
         stop("'formula' must have both left and right hand sides")
     ## need to cope with '.' in formula
     tt <- if(is.data.frame(data)) terms(formula, data=data)
@@ -91,8 +91,8 @@ ftable.formula <- function(formula, data = NULL, subset, na.action, ...)
     if(any(attr(tt, "order") > 1))
         stop("interactions are not allowed")
     ## here we do NOT want '.' expanded
-    rvars <- attr(terms(formula[-2], allowDotAsName=TRUE), "term.labels")
-    cvars <- attr(terms(formula[-3], allowDotAsName=TRUE), "term.labels")
+    rvars <- attr(terms(formula[-2L], allowDotAsName=TRUE), "term.labels")
+    cvars <- attr(terms(formula[-3L], allowDotAsName=TRUE), "term.labels")
     rhs.has.dot <- any(rvars == ".")
     lhs.has.dot <- any(cvars == ".")
     if(lhs.has.dot && rhs.has.dot)
@@ -101,7 +101,7 @@ ftable.formula <- function(formula, data = NULL, subset, na.action, ...)
     edata <- eval(m$data, parent.frame())
     if(inherits(edata, "ftable")
        || inherits(edata, "table")
-       || length(dim(edata)) > 2) {
+       || length(dim(edata)) > 2L) {
         if(inherits(edata, "ftable")) {
             data <- as.table(data)
         }
@@ -143,7 +143,7 @@ ftable.formula <- function(formula, data = NULL, subset, na.action, ...)
                                    paste(c(rvars, cvars),
                                          collapse = "+")),
                                 env = environment(formula))
-        m[[1]] <- as.name("model.frame")
+        m[[1L]] <- as.name("model.frame")
         mf <- eval(m, parent.frame())
         ftable(mf, row.vars = rvars, col.vars = cvars, ...)
     }
@@ -161,8 +161,7 @@ as.table.ftable <- function(x, ...)
                dimnames = c(xrv, xcv))
     nrv <- length(xrv)
     ncv <- length(xcv)
-    x <- aperm(x, c(seq.int(from = nrv, to = 1),
-                    seq.int(from = nrv + ncv, to = nrv + 1)))
+    x <- aperm(x, c(rev(seq_len(nrv)), rev(seq_len(ncv) + nrv)))
     class(x) <- "table"
     x
 }
@@ -203,8 +202,8 @@ format.ftable <- function(x, quote = TRUE, digits = getOption("digits"), ...)
     DATA <- rbind(if(length(xcv)) t(makeLabels(xcv)),
                   rep("", times = ncol(x)),
                   format(unclass(x), digits = digits))
-    cbind(apply(LABS, 2, format, justify = "left"),
-	  apply(DATA, 2, format, justify = "right"))
+    cbind(apply(LABS, 2L, format, justify = "left"),
+	  apply(DATA, 2L, format, justify = "right"))
 }
 
 write.ftable <- function(x, file = "", quote = TRUE, append = FALSE,
@@ -256,7 +255,7 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
     ## so there is exactly one line which does not start with a space
     ## and has n.row.vars fields (and it cannot be the first one).
     j <- i[grep("^[^[:space:]]", lines[i])]
-    if((length(j) == 1) && (j > 1)) {
+    if((length(j) == 1L) && (j > 1)) {
         ## An ftable: we can figure things out ourselves.
         n.col.vars <- j - 1
         col.vars <- vector("list", length = n.col.vars)
@@ -264,8 +263,8 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
         for(k in seq.int(from = 1, to = n.col.vars)) {
             s <- scan(file, what = "", sep = sep, quote = quote,
                       nlines = 1, quiet = TRUE)
-            col.vars[[k]] <- s[-1]
-            names(col.vars)[k] <- s[1]
+            col.vars[[k]] <- s[-1L]
+            names(col.vars)[k] <- s[1L]
         }
         row.vars <- vector("list", length = n.row.vars)
         names(row.vars) <- scan(file, what = "", sep = sep, quote =
@@ -274,7 +273,7 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
     }
     else {
         ## This is not really an ftable.
-        if((z[1] == 1) && z[2] == max(z)) {
+        if((z[1L] == 1) && z[2L] == max(z)) {
             ## Case A.  File looks like
             ##
             ##                                cvar.nam
@@ -285,12 +284,12 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
             col.vars <- vector("list", length = n.col.vars)
             s <- scan(file, what = "", sep = sep, quote = quote,
                       nlines = 2, quiet = TRUE)
-            names(col.vars) <- s[1]
-            s <- s[-1]
+            names(col.vars) <- s[1L]
+            s <- s[-1L]
             row.vars <- vector("list", length = n.row.vars)
             i <- 1 : n.row.vars
             names(row.vars) <- s[i]
-            col.vars[[1]] <- s[-i]
+            col.vars[[1L]] <- s[-i]
             z <- z[-(1 : 2)]
         }
         else {
@@ -351,5 +350,6 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
               class = "ftable")
 }
 
-as.data.frame.ftable <- function(x, row.names = NULL, optional = FALSE, ...)
+as.data.frame.ftable <-
+function(x, row.names = NULL, optional = FALSE, ...)
     as.data.frame(as.table(x), row.names, optional)
