@@ -329,10 +329,9 @@ function(f, out = "") {
  }
 
 Rd2HTML_NEWS_in_Rd <-
-function(f, out) {
+function(f, out, ...) {
     if (grepl("[.]rds$", f)) f <- readRDS(f)
-    Rd2HTML(f, out,
-            stages = c("install", "render"))
+    Rd2HTML(f, out, stages = c("install", "render"), ...)
 }
 
 Rd2pdf_NEWS_in_Rd <-
@@ -347,6 +346,10 @@ function(f, pdf_file) {
     cat("\\documentclass[", Sys.getenv("R_PAPERSIZE"), "paper]{book}\n",
         "\\usepackage[ae,hyper]{Rd}\n",
         "\\usepackage[utf8]{inputenc}\n",
+        "\\usepackage{graphicx}\n",
+        "\\setkeys{Gin}{width=0.7\\textwidth}\n",
+        "\\graphicspath{{", normalizePath(file.path(R.home("doc"), "html"), "/"),
+                            "/}}\n",
         "\\hypersetup{pdfpagemode=None,pdfstartview=FitH}\n",
         "\\begin{document}\n",
         "\\chapter*{}\\sloppy\n",
@@ -359,7 +362,7 @@ function(f, pdf_file) {
     close(out)
     od <- setwd(tempdir())
     on.exit(setwd(od))
-    texi2dvi("NEWS.tex", pdf=TRUE, quiet = TRUE)
+    texi2pdf("NEWS.tex", quiet = TRUE)
     setwd(od); on.exit()
     invisible(file.copy(file.path(tempdir(), "NEWS.pdf"),
                         pdf_file, overwrite = TRUE))
@@ -564,14 +567,30 @@ function(file)
 function(x)
 {
     .get_Rd_section_names <- function(x)
-        sapply(x, function(e) .Rd_deparse(e[[1L]]))
+        sapply(x, function(e) .Rd_get_text(e[[1L]]))
 
     do_chunk <- function(x) {
-        ## <FIXME>
-        ## This assumes that the chunk in essence is one \itemize list.
-        ## Should be safe to assume this for base R NEWS in Rd: for add
-        ## on packages we need to check ...
-        ## </FIXME>
+        ## Currently, chunks should consist of a single \itemize list
+        ## containing the news items.  Notify if there is more than one
+        ## such list, and stop if there is none.
+
+        pos <- which(RdTags(x) == "\\itemize")
+        if(!length(pos)) {
+            stop(gettextf("Malformed NEWS.Rd file:\nChunk starting\n  %s\ncontains no \\itemize.",
+                          substring(sub("^[[:space:]]*", "",
+                                        .Rd_deparse(x)),
+                                    1L, 60L)),
+                 domain = NA)
+        } else if(length(pos) > 1L) {
+            warning(gettextf("Malformed NEWS.Rd file:\nChunk starting\n  %s\ncontains more than one \\itemize.\nUsing the first one.",
+                             substring(sub("^[[:space:]]*", "",
+                                           .Rd_deparse(x)),
+                                       1L, 60L)),
+                    domain = NA)
+            pos <- pos[1L]
+        }
+        x <- x[pos]
+        
         out <- NULL
         zz <- textConnection("out", "w", local = TRUE)
         on.exit(close(zz))
@@ -579,13 +598,25 @@ function(x)
                options =
                c(Rd2txt_NEWS_in_Rd_options,
                  list(itemBullet = "\036  ")))
-        s <- gsub("^    ", "", out)
-        # Number of blanks is system dependent
-        s <- gsub("^ *\036 ", "\036", s)
+        
+        ## Try to find the column offset of the top-level bullets.
+        pat <- "^( *)\036.*"
+        off <- min(nchar(sub(pat, "\\1", out[grepl(pat, out)])))
+        pat <- sprintf("^%s\036 ",
+                       paste(rep.int(" ", off), collapse = ""))
+        s <- sub(pat, "\036", out)
+        ## Try to remove some indent for nested material.
+        pat <- sprintf("^%s",
+                       paste(rep.int(" ", off + 2L), collapse = ""))
+        s <- sub(pat, "", s)
+
         s <- paste(s, collapse = "\n")
         s <- sub("^[[:space:]]*\036", "", s)
         s <- sub("[[:space:]]*$", "", s)
-        unlist(strsplit(s, "\n\036", fixed = TRUE))
+        ## <FIXME>
+        ## Could be more fancy and use \u2022 "if possible".
+        gsub("\036", "*", unlist(strsplit(s, "\n\036", fixed = TRUE)))
+        ## </FIXME>
     }
 
     y <- x[RdTags(x) == "\\section"]
@@ -605,11 +636,7 @@ function(x)
                                                   function(e)
                                                   do_chunk(e[[2L]]))))
                            } else {
-                               pos <- which(RdTags(z) == "\\itemize")[1L]
-                               if(is.na(pos))
-                                   stop("Malformed NEWS.Rd file.")
-                               cbind(NA_character_,
-                                     do_chunk(z[pos]))
+                               cbind(NA_character_, do_chunk(z))
                            }
                        })))
 
