@@ -1085,12 +1085,13 @@ fileCompletions <- function(token)
     options <- c("add.smooth", "browser", "check.bounds", "continue",
 	"contrasts", "defaultPackages", "demo.ask", "device",
 	"digits", "dvipscmd", "echo", "editor", "encoding",
-	"example.ask", "expressions", "help.try.all.packages",
-	"htmlhelp", "HTTPUserAgent", "internet.info", "keep.source",
-	"keep.source.pkgs", "locatorBell", "mailer",
-	"max.print", "menu.graphics", "na.action", "OutDec", "pager",
-	"papersize", "par.ask.default", "pdfviewer", "pkgType",
-	"printcmd", "prompt", "repos", "scipen", "show.coef.Pvalues",
+	"example.ask", "expressions", "help.search.types",
+	"help.try.all.packages", "htmlhelp", "HTTPUserAgent",
+	"internet.info", "keep.source", "keep.source.pkgs",
+	"locatorBell", "mailer", "max.print", "menu.graphics",
+	"na.action", "OutDec", "pager", "papersize",
+	"par.ask.default", "pdfviewer", "pkgType", "printcmd",
+	"prompt", "repos", "scipen", "show.coef.Pvalues",
 	"show.error.messages", "show.signif.stars", "str",
 	"stringsAsFactors", "timeout", "ts.eps", "ts.S.compat",
 	"unzip", "verbose", "warn", "warning.length", "width")
