@@ -60,8 +60,15 @@ function(topic, package = NULL, lib.loc = NULL,
 	return(y)
     }
 
-    if(!character.only)
-	topic <- as.character(substitute(topic))
+    if(!character.only) {
+    	topic <- substitute(topic)
+    	if (is.call(topic) && (topic[[1L]] == "::" || topic[[1L]] == ":::")) {
+	    package <- as.character(topic[[2L]])
+	    topic <- as.character(topic[[3L]])
+	} else 
+	    topic <- as.character(topic)
+    }
+    
     available <- character()
     paths <- file.path(paths, "demo")
     for(p in paths) {
@@ -72,11 +79,11 @@ function(topic, package = NULL, lib.loc = NULL,
 	    available <- c(available, file.path(p, files))
     }
     if(length(available) == 0L)
-	stop(gettextf("No demo found for topic '%s'", topic), domain = NA)
+	stop(gettextf("No demo found for topic %s", sQuote(topic)), domain = NA)
     if(length(available) > 1L) {
 	available <- available[1L]
-	warning(gettextf("Demo for topic '%s' found more than once,\nusing the one found in '%s'",
-                topic, dirname(available[1L])), domain = NA)
+	warning(gettextf("Demo for topic %s' found more than once,\nusing the one found in %s",
+                sQuote(topic), sQuote(dirname(available[1L]))), domain = NA)
     }
 
     if(ask == "default")
