@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Langage for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2007   The R Development Core Team.
+ *  Copyright (C) 1998-2010   The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -62,9 +62,7 @@ Rboolean psmatch(const char *f, const char *t, Rboolean exact)
     if (exact)
 	return (Rboolean)!strcmp(f, t);
     /* else */
-    while (*f || *t) {
-	if (*t == '\0') return TRUE;
-	if (*f == '\0') return FALSE;
+    while (*t) {
 	if (*t != *f)   return FALSE;
 	t++;
 	f++;
@@ -180,28 +178,31 @@ SEXP attribute_hidden matchArgExact(SEXP tag, SEXP * list)
 
 
 /* We need to leave 'supplied' unchanged in case we call UseMethod */
-/* MULTIPLE_MATCHES was added by RI in Jan 2005 but never activated */
+/* MULTIPLE_MATCHES was added by RI in Jan 2005 but never activated:
+   code in R-2-8-branch */
 
 SEXP attribute_hidden matchArgs(SEXP formals, SEXP supplied, SEXP call)
 {
-    int i, seendots;
+    int i, seendots, arg_i = 0;
     SEXP f, a, b, dots, actuals;
-#ifdef MULTIPLE_MATCHES
-    int havedots = 0;
-#endif
 
     actuals = R_NilValue;
-    for (f = formals ; f != R_NilValue ; f = CDR(f)) {
-#ifdef MULTIPLE_MATCHES
-	if (TAG(f) ==  R_DotsSymbol) havedots = 1;
-#endif
+    for (f = formals ; f != R_NilValue ; f = CDR(f), arg_i++) {
 	actuals = CONS(R_MissingArg, actuals);
 	SET_MISSING(actuals, 1);
-	SET_ARGUSED(f, 0);
     }
+    /* We use fargused instead of ARGUSED/SET_ARGUSED on elements of
+       formals to avoid modification of the formals SEXPs.  A gc can
+       cause matchArgs to be called from finalizer code, resulting in
+       another matchArgs call with the same formals.  In R-2.10.x, this
+       corrupted the ARGUSED data of the formals and resulted in an
+       incorrect "formal argument 'foo' matched by multiple actual
+       arguments" error.
+     */
+    int fargused[arg_i];
+    memset(fargused, 0, sizeof(fargused));
 
-    for(b = supplied; b != R_NilValue; b=CDR(b))
-	SET_ARGUSED(b, 0);
+    for(b = supplied; b != R_NilValue; b = CDR(b)) SET_ARGUSED(b, 0);
 
     PROTECT(actuals);
 
@@ -211,38 +212,28 @@ SEXP attribute_hidden matchArgs(SEXP formals, SEXP supplied, SEXP call)
 
     f = formals;
     a = actuals;
+    arg_i = 0;
     while (f != R_NilValue) {
 	if (TAG(f) != R_DotsSymbol) {
 	    i = 1;
 	    for (b = supplied; b != R_NilValue; b = CDR(b)) {
 		if (TAG(b) != R_NilValue && pmatch(TAG(f), TAG(b), 1)) {
-		    if (ARGUSED(f) == 2)
-#ifdef MULTIPLE_MATCHES
-{
-			if (havedots) goto nextarg1;
-#endif
+		    if (fargused[arg_i] == 2)
 			error(_("formal argument \"%s\" matched by multiple actual arguments"),
 			      CHAR(PRINTNAME(TAG(f))));
-#ifdef MULTIPLE_MATCHES
-		    }
-#endif
 		    if (ARGUSED(b) == 2)
 			error(_("argument %d matches multiple formal arguments"), i);
 		    SETCAR(a, CAR(b));
-		    if(CAR(b) != R_MissingArg)
-			SET_MISSING(a, 0);	/* not missing this arg */
+		    if(CAR(b) != R_MissingArg) SET_MISSING(a, 0);
 		    SET_ARGUSED(b, 2);
-		    SET_ARGUSED(f, 2);
+		    fargused[arg_i] = 2;
 		}
 		i++;
-#ifdef MULTIPLE_MATCHES
-nextarg1:
-		;
-#endif
 	    }
 	}
 	f = CDR(f);
 	a = CDR(a);
+        arg_i++;
     }
 
     /* Second pass: partial matches based on tags */
@@ -253,30 +244,23 @@ nextarg1:
     seendots = 0;
     f = formals;
     a = actuals;
+    arg_i = 0;
     while (f != R_NilValue) {
-	if (ARGUSED(f) == 0) {
+	if (fargused[arg_i] == 0) {
 	    if (TAG(f) == R_DotsSymbol && !seendots) {
 		/* Record where ... value goes */
 		dots = a;
 		seendots = 1;
-	    }
-	    else {
+	    } else {
 		i = 1;
 		for (b = supplied; b != R_NilValue; b = CDR(b)) {
 		    if (ARGUSED(b) != 2 && TAG(b) != R_NilValue &&
 			pmatch(TAG(f), TAG(b), seendots)) {
 			if (ARGUSED(b))
 			    error(_("argument %d matches multiple formal arguments"), i);
-			if (ARGUSED(f) == 1)
-#ifdef MULTIPLE_MATCHES
-			{
-			    if (havedots) goto nextarg2;
-#endif
+			if (fargused[arg_i] == 1)
 			    error(_("formal argument \"%s\" matched by multiple actual arguments"),
 				  CHAR(PRINTNAME(TAG(f))));
-#ifdef MULTIPLE_MATCHES
-			}
-#endif
 			if (R_warn_partial_match_args) {
 			    warningcall(call,
 					_("partial argument match of '%s' to '%s'"),
@@ -284,21 +268,17 @@ nextarg1:
 					CHAR(PRINTNAME(TAG(f))) );
 			}
 			SETCAR(a, CAR(b));
-			if (CAR(b) != R_MissingArg)
-			    SET_MISSING(a, 0);       /* not missing this arg */
+			if (CAR(b) != R_MissingArg) SET_MISSING(a, 0);
 			SET_ARGUSED(b, 1);
-			SET_ARGUSED(f, 1);
+			fargused[arg_i] = 1;
 		    }
 		    i++;
-#ifdef MULTIPLE_MATCHES
-nextarg2:
-		    ;
-#endif
 		}
 	    }
 	}
 	f = CDR(f);
 	a = CDR(a);
+        arg_i++;
     }
 
     /* Third pass: matches based on order */
@@ -319,26 +299,22 @@ nextarg2:
 	    seendots = 1;
 	    f = CDR(f);
 	    a = CDR(a);
-	}
-	else if (CAR(a) != R_MissingArg) {
+	} else if (CAR(a) != R_MissingArg) {
 	    /* Already matched by tag */
 	    /* skip to next formal */
 	    f = CDR(f);
 	    a = CDR(a);
-	}
-	else if (ARGUSED(b) || TAG(b) != R_NilValue) {
+	} else if (ARGUSED(b) || TAG(b) != R_NilValue) {
 	    /* This value used or tagged , skip to next value */
 	    /* The second test above is needed because we */
 	    /* shouldn't consider tagged values for positional */
 	    /* matches. */
 	    /* The formal being considered remains the same */
 	    b = CDR(b);
-	}
-	else {
+	} else {
 	    /* We have a positional match */
 	    SETCAR(a, CAR(b));
-	    if(CAR(b) != R_MissingArg)
-		SET_MISSING(a, 0);
+	    if(CAR(b) != R_MissingArg) SET_MISSING(a, 0);
 	    SET_ARGUSED(b, 1);
 	    b = CDR(b);
 	    f = CDR(f);
@@ -349,29 +325,26 @@ nextarg2:
     if (dots != R_NilValue) {
 	/* Gobble up all unused actuals */
 	SET_MISSING(dots, 0);
-	i=0;
-	for(a=supplied; a!=R_NilValue ; a=CDR(a) )
-	    if(!ARGUSED(a)) i++;
+	i = 0;
+	for(a = supplied; a != R_NilValue ; a = CDR(a)) if(!ARGUSED(a)) i++;
 
 	if (i) {
 	    a = allocList(i);
 	    SET_TYPEOF(a, DOTSXP);
-	    f=a;
-	    for(b=supplied;b!=R_NilValue;b=CDR(b))
+	    f = a;
+	    for(b = supplied; b != R_NilValue; b = CDR(b))
 		if(!ARGUSED(b)) {
 		    SETCAR(f, CAR(b));
 		    SET_TAG(f, TAG(b));
-		    f=CDR(f);
+		    f = CDR(f);
 		}
 	    SETCAR(dots, a);
 	}
-    }
-    else {
+    } else {
 	/* Check that all arguments are used */
 	SEXP unused = R_NilValue, last = R_NilValue;
 	for (b = supplied; b != R_NilValue; b = CDR(b))
-	    /* Uncomment to allow unmatched empty args, as done < 2.4.0 */
-	    if (!ARGUSED(b)/* && CAR(b) != R_MissingArg) */) {
+	    if (!ARGUSED(b)) {
 		if(last == R_NilValue) {
 		    PROTECT(unused = CONS(CAR(b), R_NilValue));
 		    SET_TAG(unused, TAG(b));
@@ -384,9 +357,26 @@ nextarg2:
 	    }
 
 	if(last != R_NilValue) {
-	    errorcall(R_GlobalContext->call,
+            /* show bad arguments in call without evaluating them */
+            SEXP unusedForError = R_NilValue, last = R_NilValue;
+
+            for(b = unused ; b != R_NilValue ; b = CDR(b)) {
+                SEXP tagB = TAG(b), carB = CAR(b) ;
+                if (TYPEOF(carB) == PROMSXP) carB = PREXPR(carB) ;
+                if (last == R_NilValue) {
+                    PROTECT(last = CONS(carB, R_NilValue));
+                    SET_TAG(last, tagB);
+                    unusedForError = last;
+                } else {
+                    SETCDR(last, CONS(carB, R_NilValue));
+                    last = CDR(last);
+                    SET_TAG(last, tagB);
+                }
+            }
+	    errorcall(call /* R_GlobalContext->call */,
 		      _("unused argument(s) %s"),
-		      CHAR(STRING_ELT(deparse1line(unused, 0), 0)) + 4);
+		      CHAR(STRING_ELT(deparse1line(unusedForError, 0), 0)) + 4);
+                      /* '+ 4' is to remove 'list' from 'list(badTag1,...)' */
 	}
     }
     UNPROTECT(1);
