@@ -86,8 +86,11 @@ FILE *R_OpenSiteFile(void)
 
     fp = NULL;
     if (LoadSiteFile) {
-	if ((fp = R_fopen(getenv("R_PROFILE"), "r"))) return fp;
-	if ((fp = R_fopen(getenv("RPROFILE"), "r"))) return fp;
+	char *p = getenv("R_PROFILE");
+	if (p) {
+	    if (*p) return R_fopen(R_ExpandFileName(p), "r");
+	    else return NULL;
+	}
 #ifdef R_ARCH
 	snprintf(buf, PATH_MAX, "%s/etc/%s/Rprofile.site", R_Home, R_ARCH);
 	if ((fp = R_fopen(buf, "r"))) return fp;
@@ -101,17 +104,26 @@ FILE *R_OpenSiteFile(void)
 	/* Saving and Restoring the Global Environment */
 
 #ifndef Win32
-static char workspace_name[100] = ".RData";
+static char workspace_name[1000] = ".RData";
 
+/*
+  set_workspace_name is in src/gnuwin32/system.c and used to implement
+  drag-and-drop on Windows.
+ */
 #else
 static char workspace_name[PATH_MAX] = ".RData";
 
 void set_workspace_name(const char *fn)
 {
-    strcpy(workspace_name, fn);
+    strncpy(workspace_name, fn, PATH_MAX);
 }
 #endif
 
+const char* get_workspace_name()
+{
+    return workspace_name;
+}
+
 void R_RestoreGlobalEnv(void)
 {
     if(RestoreAction == SA_RESTORE) {
