@@ -106,7 +106,7 @@ static SEXP EnlargeVector(SEXP x, R_len_t newlen)
 
     /* Enlarge the vector itself. */
     len = length(x);
-    if (LOGICAL(GetOption(install("check.bounds"), R_BaseEnv))[0])
+    if (LOGICAL(GetOption1(install("check.bounds")))[0])
 	warning(_("assignment outside vector/list limits (extending from %d to %d)"),
 		len, newlen);
     PROTECT(x);
@@ -1163,13 +1163,13 @@ static SEXP ArrayAssign(SEXP call, SEXP x, SEXP s, SEXP y)
     return x;
 }
 
-static SEXP GetOneIndex(SEXP sub, int ind) 
+static SEXP GetOneIndex(SEXP sub, int ind)
 {
     if (ind < 0 || ind+1 > length(sub))
     	error("internal error: index %d from length %d", ind, length(sub));
     if (length(sub) > 1) {
     	switch (TYPEOF(sub)) {
-    	case INTSXP: 
+    	case INTSXP:
     	    sub = ScalarInteger(INTEGER(sub)[ind]);
     	    break;
     	case REALSXP:
@@ -1180,7 +1180,7 @@ static SEXP GetOneIndex(SEXP sub, int ind)
     	    break;
     	default:
     	    error(_("invalid subscript in list assign"));
-    	}    
+    	}
     }
     return sub;
 }
@@ -1189,22 +1189,20 @@ static SEXP GetOneIndex(SEXP sub, int ind)
 static SEXP SimpleListAssign(SEXP call, SEXP x, SEXP s, SEXP y, int ind)
 {
     SEXP indx, xi, yi, sub = CAR(s);
-    int ii, n, nx, ny, stretch=1;
+    int ii, n, nx, stretch = 1;
 
     if (length(s) > 1)
 	error(_("invalid number of subscripts to list assign"));
 
     PROTECT(sub = GetOneIndex(sub, ind));
     PROTECT(indx = makeSubscript(x, sub, &stretch, R_NilValue));
-    	
+
     n = length(indx);
     if (n > 1)
     	error(_("invalid subscript in list assign"));
 
-    ny = length(y);
     nx = length(x);
 
-
     if (stretch) {
 	SEXP t = CAR(s);
 	yi = allocList(stretch - nx);
@@ -1456,7 +1454,7 @@ SEXP attribute_hidden do_subassign2(SEXP call, SEXP op, SEXP args, SEXP rho)
 SEXP attribute_hidden
 do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP dims, indx, names, newname, subs, x, xtop, xup, y, thesub = R_NilValue;
+    SEXP dims, indx, names, newname, subs, x, xtop, xup, y, thesub = R_NilValue, xOrig = R_NilValue;
     int i, ndims, nsubs, offset, off = -1 /* -Wall */, stretch, which, len = 0 /* -Wall */;
     Rboolean S4, recursed=FALSE;
 
@@ -1495,9 +1493,10 @@ do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     /* code to allow classes to extend ENVSXP */
     if(TYPEOF(x) == S4SXP) {
+	xOrig = x; /* will be an S4 object */
         x = R_getS4DataSlot(x, ANYSXP);
-	if(x == R_NilValue)
-	  errorcall(call, _("no method for assigning subsets of this S4 class"));
+	if(TYPEOF(x) != ENVSXP)
+	  errorcall(call, _("[[<- defined for objects of type \"S4\" only for subclasses of environment"));
     }
 
     /* ENVSXP special case first */
@@ -1506,9 +1505,9 @@ do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("wrong args for environment subassignment"));
 	defineVar(install(translateChar(STRING_ELT(CAR(subs), 0))), y, x);
 	UNPROTECT(1);
-	return(x);
+	return(S4 ? xOrig : x);
     }
-    
+
     /* new case in 1.7.0, one vector index for a list, more general as of 2.10.0 */
     if (nsubs == 1) {
 	thesub = CAR(subs);
@@ -1521,7 +1520,7 @@ do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    recursed = TRUE;
 	}
     }
-    
+
     stretch = 0;
     if (isVector(x)) {
 	if (!isVectorList(x) && LENGTH(y) == 0)
@@ -1567,7 +1566,7 @@ do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 
 	which = SubassignTypeFix(&x, &y, stretch, 2, call);
-	
+
 	PROTECT(x);
 
 	switch (which) {
@@ -1723,7 +1722,7 @@ do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    }
 	    else {
 		x = SimpleListAssign(call, x, subs, y, len-1);
-	    }		
+	    }
 	}
 	else {
 	    if (ndims != nsubs)
@@ -1757,7 +1756,7 @@ do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	} else {
 	    xup = SimpleListAssign(call, xup, subs, x, len-2);
 	}
-	if (len == 2) 
+	if (len == 2)
 	    xtop = xup;
     }
     else xtop = x;
@@ -1806,7 +1805,7 @@ SEXP attribute_hidden do_subassign3(SEXP call, SEXP op, SEXP args, SEXP env)
     return R_subassign3_dflt(call, CAR(ans), nlist, CADDR(ans));
 }
 
-/* used in methods_list_dispatch.c */
+/* used in "$<-" (above) and methods_list_dispatch.c */
 SEXP R_subassign3_dflt(SEXP call, SEXP x, SEXP nlist, SEXP val)
 {
     SEXP t;
@@ -1948,7 +1947,7 @@ SEXP R_subassign3_dflt(SEXP call, SEXP x, SEXP nlist, SEXP val)
 		SET_VECTOR_ELT(x, imatch, val);
 	    }
 	    else {
-		/* We are introducing a new element. */
+		/* We are introducing a new element (=> *no* duplication) */
 		/* Enlarge the list, add the new element */
 		/* and finally, adjust the attributes. */
 		SEXP ans, ansnames;
