@@ -631,6 +631,7 @@ RxmlNanoHTTPRecv(RxmlNanoHTTPCtxtPtr ctxt)
 
 	while(1) {
 	    int maxfd = 0, howmany;
+	    struct timeval tv_save; /* select() is destructive on Linux, so save tv */
 	    R_ProcessEvents();
 #ifdef Unix
 	    if(R_wait_usec > 0) {
@@ -648,6 +649,7 @@ RxmlNanoHTTPRecv(RxmlNanoHTTPCtxtPtr ctxt)
 	    tv.tv_usec = 0;
 #endif
 
+	    tv_save = tv;
 
 #ifdef Unix
 	    maxfd = setSelectMask(R_InputHandlers, &rfd);
@@ -666,7 +668,7 @@ RxmlNanoHTTPRecv(RxmlNanoHTTPCtxtPtr ctxt)
 		return(0);
 	    }
 	    if (howmany == 0) {
-		used += tv.tv_sec + 1e-6 * tv.tv_usec;
+		used += tv_save.tv_sec + 1e-6 * tv_save.tv_usec;
 		if(used >= timeout) return(0);
 		continue;
 	    }
@@ -990,6 +992,7 @@ RxmlNanoHTTPConnectAttempt(struct sockaddr *addr)
 
     while(1) {
 	int maxfd = 0;
+	struct timeval tv_save; /* select() is destructive on Linux, so save tv */
 	R_ProcessEvents();
 #ifdef Unix
 	if(R_wait_usec > 0) {
@@ -1007,6 +1010,7 @@ RxmlNanoHTTPConnectAttempt(struct sockaddr *addr)
 	tv.tv_usec = 0;
 #endif
 
+	tv_save = tv;
 
 #ifdef Unix
 	maxfd = setSelectMask(R_InputHandlers, &rfd);
@@ -1022,7 +1026,7 @@ RxmlNanoHTTPConnectAttempt(struct sockaddr *addr)
 	case 0:
 	    /* Time out */
 	    RxmlMessage(0, "Connect attempt timed out");
-	    used += tv.tv_sec + 1e-6 * tv.tv_usec;
+	    used += tv_save.tv_sec + 1e-6 * tv_save.tv_usec;
 	    if(used < timeout) continue;
 	    closesocket(s);
 	    return(-1);
